import gleam/dynamic/decode
import gleam/json

pub type NewUserState {
  NewUserState
}

pub type NewUserClientMessages {
  Register(game_code: String, username: String)
  CreateRoom(game_code: String, username: String)
}

pub fn new_user_client_messages_decoder() -> decode.Decoder(
  NewUserClientMessages,
) {
  use variant <- decode.field("type", decode.string)
  case variant {
    "Register" -> {
      use game_code <- decode.field("game_code", decode.string)
      use username <- decode.field("username", decode.string)
      decode.success(Register(game_code:, username:))
    }
    "CreateRoom" -> {
      use game_code <- decode.field("game_code", decode.string)
      use username <- decode.field("username", decode.string)
      decode.success(CreateRoom(game_code:, username:))
    }
    _ -> decode.failure(Register("", ""), "NewUserClientMessages")
  }
}

pub type NewUserServerMessages {
  RegisterResponse(username: String, token: String)
  HostRegisterResponse(username: String, token: String)
}

pub fn encode_new_user_server_messages(
  new_user_server_messages: NewUserServerMessages,
) -> json.Json {
  case new_user_server_messages {
    RegisterResponse(..) ->
      json.object([
        #("type", json.string("RegisterResponse")),
        #("username", json.string(new_user_server_messages.username)),
        #("token", json.string(new_user_server_messages.token)),
      ])
    HostRegisterResponse(..) ->
      json.object([
        #("type", json.string("HostRegisterResponse")),
        #("username", json.string(new_user_server_messages.username)),
        #("token", json.string(new_user_server_messages.token)),
      ])
  }
}
